import React, { useState, useCallback, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, arc, pie, timer } from 'd3';

const csvUrl =
  'https://gist.githubusercontent.com/curran/b236990081a24761f7000567094914e0/raw/acd2b8cecfe51c520622fbaf407ee88b8796bfc6/cssNamedColors.csv';

const width = 960;
const height = 500;
const centerX = width / 2;
const centerY = height / 2 - 60;
const move = 110;

const pieArc1 = arc()
  .innerRadius(110)
  .outerRadius(120);

const pieArc2 = arc()
  .innerRadius(130)
  .outerRadius(140);

const pieArc3 = arc()
  .innerRadius(150)
  .outerRadius(160);

const App = () => {
  const [data, setData] = useState(null);

  useEffect(() => {
    d3.csv(csvUrl).then(setData);
  }, []);
  
  const [rotation, setRotation] = useState(0);
  useEffect(() => {
    const rotateTimer = timer(time => setRotation(time * 0.01));
    return () => rotateTimer.stop();
  }, []);

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const colorPie = pie().value(2);

  return (
    <svg width={width} height={height}>
      <g transform={`translate(${centerX},${centerY}) rotate(${rotation})`}>
        {colorPie(data).map(d => (
          <path fill={d.data['RGB hex value']} d={pieArc1(d)} />
        ))}
      </g>

      <g transform={`translate(${centerX},${centerY})`}>
        {colorPie(data).map(d => (
          <path fill={d.data['RGB hex value']} d={pieArc2(d)} />
        ))}
      </g>

      <g transform={`translate(${centerX},${centerY}) rotate(${-rotation})`}>
        {colorPie(data).map(d => (
          <path fill={d.data['RGB hex value']} d={pieArc3(d)} />
        ))}
      </g>
      <g transform={`translate(${centerX + move},${centerY + move}) rotate(${rotation})`}>
        {colorPie(data).map(d => (
          <path fill={d.data['RGB hex value']} d={pieArc1(d)} />
        ))}
      </g>
      <g transform={`translate(${centerX + move},${centerY + move})`}>
        {colorPie(data).map(d => (
          <path fill={d.data['RGB hex value']} d={pieArc2(d)} />
        ))}
      </g>
      <g transform={`translate(${centerX + move},${centerY + move}) rotate(${-rotation})`}>
        {colorPie(data).map(d => (
          <path fill={d.data['RGB hex value']} d={pieArc3(d)} />
        ))}
      </g>
      <g transform={`translate(${centerX - move},${centerY + move}) rotate(${rotation})`}>
        {colorPie(data).map(d => (
          <path fill={d.data['RGB hex value']} d={pieArc1(d)} />
        ))}
      </g>
      <g transform={`translate(${centerX - move},${centerY + move})`}>
        {colorPie(data).map(d => (
          <path fill={d.data['RGB hex value']} d={pieArc2(d)} />
        ))}
      </g>
      <g transform={`translate(${centerX - move},${centerY + move}) rotate(${-rotation})`}>
        {colorPie(data).map(d => (
          <path fill={d.data['RGB hex value']} d={pieArc3(d)} />
        ))}
      </g>
    </svg>
  );
};
const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
